@@ -86,10 +86,11 @@ static uint64_t _registry_hash_string(const char * str) {
 static _infix_registry_entry_t * _registry_lookup(infix_registry_t * registry, const char * name) {
     if (!registry || !name)
         return nullptr;
-    size_t index = _registry_hash_string(name) % registry->num_buckets;
+    uint64_t hash = _registry_hash_string(name);
+    size_t index = hash % registry->num_buckets;
     // Traverse the linked list (chain) at the computed bucket index.
     for (_infix_registry_entry_t * current = registry->buckets[index]; current; current = current->next)
-        if (strcmp(current->name, name) == 0)
+        if (current->hash == hash && strcmp(current->name, name) == 0)
             return current;
     return nullptr;
 }
@@ -98,18 +99,14 @@ static _infix_registry_entry_t * _registry_lookup(infix_registry_t * registry, c
  * @internal
  * @brief Expands the registry's hash table and redistributes existing entries.
  * @details This is triggered when the load factor exceeds 0.75 to maintain O(1) lookup performance.
- *          The new bucket array is allocated from the registry's arena.
+ *          The new bucket array is allocated using `infix_calloc` and the old one is freed.
  */
 static void _registry_rehash(infix_registry_t * registry) {
     // Roughly double the size. Keep it odd to slightly help with distribution.
     size_t new_num_buckets = registry->num_buckets * 2 + 1;
 
-    // Allocate new buckets from the arena.
-    // Note: The old bucket array remains allocated in the arena (leak within arena),
-    // but this is an acceptable trade-off for the simplicity of arena management
-    // and the fact that registries usually grow during initialization and persist.
-    _infix_registry_entry_t ** new_buckets = infix_arena_calloc(
-        registry->arena, new_num_buckets, sizeof(_infix_registry_entry_t *), _Alignof(_infix_registry_entry_t *));
+    // Allocate new buckets from the heap.
+    _infix_registry_entry_t ** new_buckets = infix_calloc(new_num_buckets, sizeof(_infix_registry_entry_t *));
 
     if (!new_buckets) {
         // If allocation fails, we simply return and continue using the existing buckets.
@@ -123,9 +120,8 @@ static void _registry_rehash(infix_registry_t * registry) {
         while (entry) {
             _infix_registry_entry_t * next_entry = entry->next;  // Save next pointer
 
-            // Calculate new index
-            uint64_t hash = _registry_hash_string(entry->name);
-            size_t new_index = hash % new_num_buckets;
+            // Use the pre-calculated hash
+            size_t new_index = entry->hash % new_num_buckets;
 
             // Insert into new bucket (prepend)
             entry->next = new_buckets[new_index];
@@ -135,7 +131,8 @@ static void _registry_rehash(infix_registry_t * registry) {
         }
     }
 
-    // Update the registry to use the new table.
+    // Update the registry to use the new table and free the old one.
+    infix_free(registry->buckets);
     registry->buckets = new_buckets;
     registry->num_buckets = new_num_buckets;
 }
@@ -159,7 +156,8 @@ static _infix_registry_entry_t * _registry_insert(infix_registry_t * registry, c
     if (registry->num_items * 4 > registry->num_buckets * 3)
         _registry_rehash(registry);
 
-    size_t index = _registry_hash_string(name) % registry->num_buckets;
+    uint64_t hash = _registry_hash_string(name);
+    size_t index = hash % registry->num_buckets;
     _infix_registry_entry_t * new_entry =
         infix_arena_alloc(registry->arena, sizeof(_infix_registry_entry_t), _Alignof(_infix_registry_entry_t));
     if (!new_entry) {
@@ -174,6 +172,7 @@ static _infix_registry_entry_t * _registry_insert(infix_registry_t * registry, c
     }
     infix_memcpy((void *)name_copy, name, name_len);
     new_entry->name = name_copy;
+    new_entry->hash = hash;
     new_entry->type = nullptr;
     new_entry->is_forward_declaration = false;
     // Prepend to the linked list in the bucket.
@@ -209,8 +208,7 @@ INFIX_API c23_nodiscard infix_registry_t * infix_registry_create(void) {
     }
     registry->is_external_arena = false;  // Mark that this arena is owned by the registry.
     registry->num_buckets = INITIAL_REGISTRY_BUCKETS;
-    registry->buckets = infix_arena_calloc(
-        registry->arena, registry->num_buckets, sizeof(_infix_registry_entry_t *), _Alignof(_infix_registry_entry_t *));
+    registry->buckets = infix_calloc(registry->num_buckets, sizeof(_infix_registry_entry_t *));
     if (!registry->buckets) {
         infix_arena_destroy(registry->arena);
         infix_free(registry);
@@ -246,8 +244,7 @@ INFIX_API c23_nodiscard infix_registry_t * infix_registry_create_in_arena(infix_
     registry->arena = arena;
     registry->is_external_arena = true;  // Mark the arena as user-owned
     registry->num_buckets = INITIAL_REGISTRY_BUCKETS;
-    registry->buckets = infix_arena_calloc(
-        registry->arena, registry->num_buckets, sizeof(_infix_registry_entry_t *), _Alignof(_infix_registry_entry_t *));
+    registry->buckets = infix_calloc(registry->num_buckets, sizeof(_infix_registry_entry_t *));
     if (!registry->buckets) {
         infix_free(registry);
         _infix_set_error(INFIX_CATEGORY_ALLOCATION, INFIX_CODE_OUT_OF_MEMORY, 0);
@@ -313,6 +310,8 @@ INFIX_API void infix_registry_destroy(infix_registry_t * registry) {
     // Only destroy the arena if it was created internally by `infix_registry_create`.
     if (!registry->is_external_arena)
         infix_arena_destroy(registry->arena);
+    // Free the bucket array which is now allocated from the heap.
+    infix_free(registry->buckets);
     // Always free the registry struct itself.
     infix_free(registry);
 }
@@ -478,6 +477,8 @@ static void _registry_parser_skip_whitespace(_registry_parser_state_t * state) {
 static char * _registry_parser_parse_name(_registry_parser_state_t * state, char * buffer, size_t buf_size) {
     _registry_parser_skip_whitespace(state);
     const char * name_start = state->p;
+    if (!isalpha((unsigned char)*name_start) && *name_start != '_')
+        return nullptr;
     while (isalnum((unsigned char)*state->p) || *state->p == '_' || *state->p == ':') {
         if (*state->p == ':' && state->p[1] != ':')
             break;  // Handle single colon as non-identifier char.
@@ -653,31 +654,78 @@ INFIX_API c23_nodiscard infix_status infix_register_types(infix_registry_t * reg
         if (*state.p == ';')
             state.p++;
     }
+
+    // Use a single temporary arena for ALL definitions in this batch to reduce malloc pressure.
+    infix_arena_t * parser_arena = infix_arena_create(65536);
+    if (!parser_arena) {
+        _infix_set_error(INFIX_CATEGORY_ALLOCATION, INFIX_CODE_OUT_OF_MEMORY, 0);
+        final_status = INFIX_ERROR_ALLOCATION_FAILED;
+        goto cleanup;
+    }
+
     // Pass 2: Parse the bodies of all found definitions into the registry.
     for (size_t i = 0; i < num_defs_found; ++i) {
         _infix_registry_entry_t * entry = defs_found[i].entry;
-        // Make a temporary, null-terminated copy of the definition body substring.
-        char * body_copy = infix_malloc(defs_found[i].def_body_len + 1);
-        if (!body_copy) {
-            _infix_set_error(INFIX_CATEGORY_ALLOCATION, INFIX_CODE_OUT_OF_MEMORY, 0);
-            final_status = INFIX_ERROR_ALLOCATION_FAILED;
-            goto cleanup;
+
+        // FAST-PATH: Check if the definition body is just a single primitive keyword.
+        // This avoids the overhead of the recursive-descent parser for simple aliases.
+        infix_type * fast_primitive = nullptr;
+        char body_buffer[64];
+        if (defs_found[i].def_body_len > 0 && defs_found[i].def_body_len < sizeof(body_buffer)) {
+            infix_memcpy(body_buffer, defs_found[i].def_body_start, defs_found[i].def_body_len);
+            body_buffer[defs_found[i].def_body_len] = '\0';
+
+            // Use a temporary parser state for parse_primitive.
+            parser_state fast_state = {.p = body_buffer, .start = body_buffer, .arena = parser_arena, .depth = 0};
+            fast_primitive = parse_primitive(&fast_state);
+
+            // Ensure no trailing junk even in fast-path.
+            if (fast_primitive) {
+                skip_whitespace(&fast_state);
+                if (*fast_state.p != '\0')
+                    fast_primitive = nullptr;
+            }
         }
-        infix_memcpy(body_copy, defs_found[i].def_body_start, defs_found[i].def_body_len);
-        body_copy[defs_found[i].def_body_len] = '\0';
-        // "Parse" step: parse into a temporary arena.
+
         infix_type * raw_type = nullptr;
-        infix_arena_t * parser_arena = nullptr;
-        infix_status status = _infix_parse_type_internal(&raw_type, &parser_arena, body_copy);
-        infix_free(body_copy);
-        if (status != INFIX_SUCCESS) {
-            // Adjust the error position to be relative to the full definition string.
-            infix_error_details_t err = infix_get_last_error();
-            _infix_set_error(err.category, err.code, (defs_found[i].def_body_start - definitions) + err.position);
-            final_status = INFIX_ERROR_INVALID_ARGUMENT;
-            infix_arena_destroy(parser_arena);
-            goto cleanup;
+        if (fast_primitive) {
+            raw_type = fast_primitive;
         }
+        else {
+            // SLOW-PATH: Fall back to full signature parser.
+            // Make a temporary, null-terminated copy of the definition body substring.
+            char * body_copy = infix_malloc(defs_found[i].def_body_len + 1);
+            if (!body_copy) {
+                _infix_set_error(INFIX_CATEGORY_ALLOCATION, INFIX_CODE_OUT_OF_MEMORY, 0);
+                final_status = INFIX_ERROR_ALLOCATION_FAILED;
+                infix_arena_destroy(parser_arena);
+                goto cleanup;
+            }
+            infix_memcpy(body_copy, defs_found[i].def_body_start, defs_found[i].def_body_len);
+            body_copy[defs_found[i].def_body_len] = '\0';
+
+            parser_state state = {.p = body_copy, .start = body_copy, .arena = parser_arena, .depth = 0};
+            raw_type = parse_type(&state);
+
+            if (raw_type) {
+                skip_whitespace(&state);
+                if (*state.p != '\0') {
+                    _infix_set_parser_error(&state, INFIX_CODE_UNEXPECTED_TOKEN);
+                    raw_type = nullptr;
+                }
+            }
+
+            if (!raw_type) {
+                infix_error_details_t err = infix_get_last_error();
+                _infix_set_error(err.category, err.code, (defs_found[i].def_body_start - definitions) + err.position);
+                final_status = INFIX_ERROR_INVALID_ARGUMENT;
+                infix_free(body_copy);
+                infix_arena_destroy(parser_arena);
+                goto cleanup;
+            }
+            infix_free(body_copy);
+        }
+
         const infix_type * type_to_alias = raw_type;
         // If the RHS is another named type (e.g., @MyAlias = @ExistingType),
         // we need to resolve it first to get the actual type we're aliasing.
@@ -711,11 +759,10 @@ INFIX_API c23_nodiscard infix_status infix_register_types(infix_registry_t * reg
         else  // Dynamic type: deep copy.
             new_def = _copy_type_graph_to_arena(registry->arena, type_to_alias);
 
-        infix_arena_destroy(parser_arena);  // The temporary raw_type is no longer needed.
-
         if (!new_def) {
             _infix_set_error(INFIX_CATEGORY_ALLOCATION, INFIX_CODE_OUT_OF_MEMORY, 0);
             final_status = INFIX_ERROR_ALLOCATION_FAILED;
+            infix_arena_destroy(parser_arena);
             goto cleanup;
         }
 
@@ -738,6 +785,9 @@ INFIX_API c23_nodiscard infix_status infix_register_types(infix_registry_t * reg
         entry->type->name = entry->name;
         entry->is_forward_declaration = false;
     }
+
+    infix_arena_destroy(parser_arena);
+
     // Pass 3: Resolve and layout all the newly defined types.
     for (size_t i = 0; i < num_defs_found; ++i) {
         _infix_registry_entry_t * entry = defs_found[i].entry;
@@ -893,4 +943,4 @@ INFIX_API c23_nodiscard const infix_type * infix_registry_lookup_type(const infi
     if (entry && entry->type && !entry->is_forward_declaration)
         return entry->type;
     return nullptr;
-}
+}
\ No newline at end of file
