@@ -111,6 +111,7 @@ typedef enum {
     OP_PUSH_SINT64,
     OP_PUSH_UINT64,
     OP_PUSH_FLOAT,
+    OP_PUSH_FLOAT16,
     OP_PUSH_DOUBLE,
     OP_PUSH_LONGDOUBLE,
     OP_PUSH_SINT128,
@@ -138,6 +139,7 @@ typedef enum {
     OP_RET_SINT64,
     OP_RET_UINT64,
     OP_RET_FLOAT,
+    OP_RET_FLOAT16,
     OP_RET_DOUBLE,
     OP_RET_LONGDOUBLE,
     OP_RET_SINT128,
